<?php
/**
 * @package   awf
 * @copyright Copyright (c)2014-2022 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU GPL version 3 or later
 */

namespace Awf\Tests\Download\Adapter;

use Awf\Download\Adapter\Fopen;
use Awf\Tests\Helpers\AwfTestCase;
use Awf\Tests\Helpers\Download\FakeFopen;
use Awf\Tests\Helpers\ReflectionHelper;

require_once __DIR__ . '/../../Helpers/Download/FakeFopenImporter.php';
require_once __DIR__ . '/FopenDataprovider.php';

/**
 * @covers  Awf\Download\Adapter\Fopen::<protected>
 * @covers  Awf\Download\Adapter\Fopen::<private>
 */
class FopenTest extends AwfTestCase
{
	public static function setUpBeforeClass()
	{
		global $awfTest_FakeFopen_Active;
        $awfTest_FakeFopen_Active = true;

		parent::setUpBeforeClass();
	}

	public static function tearDownAfterClass()
	{
		global $awfTest_FakeFopen_Active;
        $awfTest_FakeFopen_Active = false;

		parent::tearDownAfterClass();
	}

	/**
	 * @covers  Awf\Download\Adapter\Fopen::__construct
	 */
	public function testConstructor()
	{
		$adapter = new Fopen();

		$this->assertInstanceOf('Awf\\Download\\Adapter\\Fopen', $adapter, 'Adapter must match correct object type');
		$this->assertEquals(100, ReflectionHelper::getValue($adapter, 'priority'), 'Adapter priority must match');
		$this->assertEquals(false, ReflectionHelper::getValue($adapter, 'supportsFileSize'), 'Adapter must not support file size');
		$this->assertEquals(true, ReflectionHelper::getValue($adapter, 'supportsChunkDownload'), 'Adapter must support chunked download');
		$this->assertEquals('fopen', ReflectionHelper::getValue($adapter, 'name'), 'Adapter must have the correct name');
		$this->assertEquals(true, ReflectionHelper::getValue($adapter, 'isSupported'), 'Adapter must be supported');
	}

	/**
	 * @covers          Awf\Download\Adapter\Fopen::downloadAndReturn
	 * @dataProvider    FopenDataprovider::getTestDownloadAndReturn
	 */
	public function testDownloadAndReturn(array $config, array $test)
	{
		FakeFopen::setUp($config);
		$adapter = new Fopen();

		if ($test['exception'] !== false)
		{
			$this->setExpectedException($test['exception']['name'], $test['exception']['message'], $test['exception']['code']);
		}

		$ret = $adapter->downloadAndReturn($test['url'], $test['from'], $test['to']);
		$retSize = 0;

		if (is_string($ret))
		{
			$retSize = strlen($ret);
		}

		$this->assertEquals($test['retSize'], $retSize, $test['message']);
	}
}
